@@ -200,13 +200,27 @@ def specificity(actual, pred, pos_label=0):
     return recall_score(actual, pred, pos_label=pos_label)
 
 
-def objective(trial, df, study_name, stage, sensor, use_average, model_mode, suggest):
+def objective(trial, df, study_name, stage, sensor, use_average, model_mode, suggest, custom_features, remote_tracking):
 
-        # mlflow.set_experiment('raptor_exp')
+        mlflow.set_experiment(study_name)
 
         # artifact_path = mlflow.get_artifact_uri()
 
-        mlflow.start_run(nested=True)
+        mlflow.start_run()
+
+        if remote_tracking:
+                mlflow.set_tracking_uri(
+            # "http://ec2-13-40-214-238.eu-west-2.compute.amazonaws.com:5000/"
+            # "http://ec2-13-40-86-186.eu-west-2.compute.amazonaws.com:5000/"
+            "http://ec2-3-10-175-206.eu-west-2.compute.amazonaws.com:5000/"
+            )
+        else:
+            mlflow.set_tracking_uri("http://localhost:5000")
+    #     uri = os.path.join(os.getcwd(), "sqlite:///mlruns.db")
+    #     mlflow.set_tracking_uri(uri)
+
+
+
     # with mlflow.start_run():
         if suggest:
             stage = trial.suggest_categorical("stages", ["pause", "all", 'random'])
@@ -215,7 +229,16 @@ def objective(trial, df, study_name, stage, sensor, use_average, model_mode, sug
             model_mode = trial.suggest_categorical("model_mode", ["random", "compare"])
 
 
-        data, sensors_to_use, stages_to_use = get_train_features(df, sensor=sensor, stage=stage, use_average=use_average)
+
+        if custom_features:
+            data = df
+            sensors_to_use, stages_to_use = df.columns, None
+
+        else:
+            data, sensors_to_use, stages_to_use = get_train_features(df, sensor=sensor, stage=stage, use_average=use_average)
+        # except:
+        #     data = df
+        #     sensors_to_use, stages_to_use = df.columns, None
 
 
         clf = setup(data, target = 'result', silent=True, log_plots=True, experiment_name=study_name, preprocess=False)
@@ -284,9 +307,12 @@ def objective(trial, df, study_name, stage, sensor, use_average, model_mode, sug
 
         mlflow.log_metrics(metrics)
 
-        mlflow.set_tag('sensor_cols', sensors_to_use)
-        mlflow.set_tag('stage', stages_to_use)
-        mlflow.set_tag('use_average', use_average)
+        try:
+            mlflow.set_tag('sensor_cols', sensors_to_use)
+            mlflow.set_tag('stage', stages_to_use)
+            mlflow.set_tag('use_average', use_average)
+        except:
+            pass
 
         mlflow.set_tag('model_name', model_name)
         mlflow.set_tag('model_params', model_params)
@@ -294,15 +320,16 @@ def objective(trial, df, study_name, stage, sensor, use_average, model_mode, sug
         mlflow.set_tag('model_mode', model_mode)
         mlflow.set_tag('suggest', suggest)
 
-        mlflow.end_run()
         print('Click on this link to track experiments: ', mlflow.get_tracking_uri())
+        mlflow.end_run()
 
 
 
 
 
 
-def train_experiments(df, study_name='raptor', direction='maximize', stage='pause', sensor='12', use_average=False, model_mode='random', n_trials=5, suggest=False, remote_tracking=True):
+
+def train_experiments(df, study_name='raptor', direction='maximize', stage='pause', sensor='12', use_average=False, model_mode='random', n_trials=5, custom_features=False, suggest=False, remote_tracking=True):
     """trains several models during different trials and logs them. Experiemnt can be tracked on "http://ec2-3-10-175-206.eu-west-2.compute.amazonaws.com:5000/"
 
 
@@ -318,16 +345,7 @@ def train_experiments(df, study_name='raptor', direction='maximize', stage='paus
         suggest (bool, optional): if True, suggest features, stage, model mode to use for training. Defaults to False.
     """    
     
-    if remote_tracking:
-        mlflow.set_tracking_uri(
-        # "http://ec2-13-40-214-238.eu-west-2.compute.amazonaws.com:5000/"
-        # "http://ec2-13-40-86-186.eu-west-2.compute.amazonaws.com:5000/"
-        "http://ec2-3-10-175-206.eu-west-2.compute.amazonaws.com:5000/"
-        )
-    else:
-        mlflow.set_tracking_uri("http://localhost:5000")
-    #     uri = os.path.join(os.getcwd(), "sqlite:///mlruns.db")
-    #     mlflow.set_tracking_uri(uri)
+    
 
 
     # Initialize client
@@ -337,7 +355,7 @@ def train_experiments(df, study_name='raptor', direction='maximize', stage='paus
     # print(mlflow.get_tracking_uri())
 
     study = optuna.create_study(study_name=study_name, direction=direction)
-    study.optimize(lambda trial: objective(trial, df, study_name, stage, sensor, use_average, model_mode, suggest), n_trials=n_trials)
+    study.optimize(lambda trial: objective(trial, df, study_name, stage, sensor, use_average, model_mode, suggest, custom_features, remote_tracking), n_trials=n_trials)
 
     # return mlflow.get_artifact_uri(), mlflow.get_tracking_uri()
         
@@ -375,3 +393,9 @@ def train_experiments(df, study_name='raptor', direction='maximize', stage='paus
 
     # study = optuna.create_study(study_name='raptor', direction="maximize")
     # study.optimize(lambda trial: objective(trial, stage, sensor, use_average, model_mode, suggest), n_trials=n_trials)
+
+
+
+
+
+
